//driver.cpp - Driver for "Antlions"

#include <iostream>
#include <Windows.h>
#include <conio.h>
#include <stdio.h>
#include <time.h>
#include <string>
#include "Organism.h"
#include "World.h"
#include "Ant.h"
#include "Lion.h"
using namespace std;

//Timer
//http://www.cplusplus.com/forum/beginner/317/
class Timer {
	private:
		//when the clock began
		unsigned long begTime;
	public:
		//start the timer
		//(meaning take the current reading from the clock and save it)
		void start() {
			begTime = clock();
		}

		//how much time has passed?
		unsigned long elapsedTime() {
			return ((unsigned long) clock() - begTime) / CLOCKS_PER_SEC;
		}

		//have we passed n seconds?
		bool isTimeout(unsigned long seconds) {
			return seconds >= elapsedTime();
		}
};

//main method fired when program loads
int main()
{
    //bright white text (the norm for us)
	//if nothing for background is specified, it's black
	//(oddly enough, same for foreground - defaults to black, not white or gray)
	SetConsoleTextAttribute(GetStdHandle( STD_OUTPUT_HANDLE ), FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY);
	
	//Welcome!
	cout << "Welcome to \"Antlions\"." << endl;
	cout << "Based on a concept by David Russell." << endl;
	cout << "Implemented by William Matheson." << endl;
	cout << "Special thanks to Jayd Newell for helping me find my silly mistakes." << endl;
	cout << "Consult README.TXT for simulation design and rules." << endl << endl;

	cout << "Press Enter to display the grid." << endl;
	cin.get();
	if (cin.eof()) //if Ctrl+Z is inputted
	{
		cout << "Goodbye!" << endl;
		return -1;
	}
	
	//initialize a world
	World myWorld = World();
	
	//legend
	cout << "Inital configuration." << endl;
    SetConsoleTextAttribute(GetStdHandle( STD_OUTPUT_HANDLE ), FOREGROUND_GREEN | FOREGROUND_INTENSITY);
	cout << "O";//green
    SetConsoleTextAttribute(GetStdHandle( STD_OUTPUT_HANDLE ), FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY);
	cout << " represents ants; ";
	SetConsoleTextAttribute(GetStdHandle( STD_OUTPUT_HANDLE ), FOREGROUND_RED | FOREGROUND_INTENSITY);
    cout << "X";//red
	SetConsoleTextAttribute(GetStdHandle( STD_OUTPUT_HANDLE ), FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY);
	cout << " represents antlions (they eat ants!)" << endl << endl;

	//display the world
	cout << myWorld;

	cout << endl << "Press Enter to begin the simulation." << endl;
	cin.get();
	if (cin.eof()) //if Ctrl+Z is inputted
	{
		cout << "Goodbye!" << endl;
		return -1;
	}
	while (_kbhit()) {
    _getch(); //clear buffer so _kbhit can still work
    };

	//iteration counter
	int num = 1;

	//run and display first iteration
	//clear screen
	//portable way:
	//cout << string(50, '\n');
	//snappy way (not like the color stuff is portable anyway! j/k):
	system("cls");//apparently system() is not good to use in interviews with Microsoft:
	//http://www.cplusplus.com/forum/beginner/5830/#msg28409
	cout << endl << endl;
	cout << "Iteration number " << num << ":" << endl;
	myWorld.run(); //run
    cout << myWorld; //display
	cout << "Press Enter to stop." << endl;
	num ++;
	
	//display new iteration every second or until key is pressed
	Timer t;//Timer object
	while (!_kbhit() && num <= 10000)//until user presses a key or 10,000 iterations pass (about 3 hours)
	{
		t.start();
	    while(true) {
		if(t.elapsedTime() >= 1) {//only whole values work - 0 for superfast, 1 for one second, etc..
			//run another turn and display it
			system("cls");
			cout << endl << endl;
			cout << "Iteration number " << num << ":" << endl;
			myWorld.run();
			cout << myWorld;
			cout << "Press Enter to stop." << endl;
			num ++;
			break;
		}
		else {//want to be able to quit w/o waiting for next iteration
			if (_kbhit())
				break;
		}
		}
	}

	cin.ignore();//so our next cin.get() can pause
	cout << "Thanks for viewing! (Press Enter to exit.)";
	cin.get();
	return 0;
}